<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2018 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Hilfsklasse für die Verarbeitung von GET, POST, COOKIE und SESSION Daten.
 *
 * @package GhCore
 * @subpackage globals
 * @category Hilfsklassen
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2018 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhCoreGlobals
{
  /**
   * Legt fest, ob das mod_rewrite Modul verwendet werden soll.
   * @var boolean
   */
  public static $useModRewrite = false;

  /**
   * Liefert einen Variablenwert aus GET-/POST-Requests, Cookies oder Session.
   *
   * Es können mehrere Datenquellen (GET, POST, COOKIE, SESSION) nach der
   * Variable durchsucht werden. Die Art und Reihenfolge der Datenquellen
   * kann als Parameter $methods übergeben werden (in Form einer Zeichenkette,
   * in der 'G' für GET, 'P' für POST, 'C' für COOKIE und 'S' für SESSION
   * steht). Die angegebenen Datenquellen werden in der Reihenfolge durchsucht,
   * und die erste Quelle, in der die Variable existiert liefert den Wert.
   *
   * Die Voreinstellung für die Datenquellen ist, nur GET und POST zu
   * durchsuchen. Hierbei wird die ermittelte Request-Methode (aus dem Wert
   * $_SERVER['REQUEST_METHOD']) verwendet. Ist diese nicht verfügbar, wird
   * erst POST, dann GET durchsucht. Cookies und Session werden in der
   * Voreinstellung nicht berücksichtigt.
   *
   * Wird die Variable in keiner der angegebenen Datenquellen gefunden, so wird
   * der Vorgabewert zurückgegeben (Voreinstellung: null).
   *
   * @param string $name    Name der Variablen
   * @param string $default Vorgabewert, falls kein Wert existiert
   * @param string $methods Reihenfolge der Datenquellen
   *  ('G': GET, 'P': POST, 'C': COOKIE, 'S': SESSION)
   * @return string|array
   */
  public static function getValue($name, $default = null, $methods = null)
  {
    $functions = array('G' => 'get', 'P' => 'post', 'C' => 'cookie', 'S' => 'session');

    if($methods === null)
    {
      if(isset($_SERVER['REQUEST_METHOD']) && strtolower($_SERVER['REQUEST_METHOD']) == 'get')
        $methods = 'GP';
      else
        $methods = 'PG';
    }

    $value = null;
    for($i=0; $i<strlen($methods); $i++)
    {
      $function = $functions[strtoupper($methods[$i])];
      if(!$function)
        continue;
      $value = call_user_func_array(array(__CLASS__, $function), array($name, null));
      if($value !== null)
        return $value;
    }

    return $default;
  }

  /**
   * Erstellt Query-Strings auf Basis des aktuellen Query-Strings der Webseite.
   *
   * Beispiel:
   * $uri = GhCoreGlobals::get_query($exclude, 'var1', 'var2');
   *
   * Falls $exclude true ist, werden alle nachfolgenden Variablen aus dem neuen Query-String herausgelöscht.
   * Falls $exclude false ist, werden nur die nachfolgenden Variablen in den neuen Query-Strings übernommen.
   *
   * @param boolean $exclude true: Variablen entfernen, false: Variablen übernehmen
   * @return string
   */
  public static function getQuery($exclude = true)
  {
    $queryvars = $exclude ? $_GET : array();

    for($i = 1, $n = func_num_args(); $i < $n; $i++)
    {
      if(($pos = strpos(func_get_arg($i), '=')) === false)
      {
        $key = func_get_arg($i);

        if($exclude && isset($queryvars[$key]))
          unset($queryvars[$key]);
        else if(GhCoreStrings::realTrim(GhCoreGlobals::get($key)) != '')
          $queryvars[$key] = trim(GhCoreGlobals::get($key));
      }
      else
      {
        $key   = substr(func_get_arg($i), 0, $pos);
        $value = substr(func_get_arg($i), $pos + 1);

        $queryvars[$key] = $value;
      }
    }

    if(ini_get("session.use_only_cookies") == 0)
    {
      if($_COOKIE(session_name()) == NULL)
        $queryvars[session_name()] = session_id();
      else if(isset($queryvars[session_name()]))
        unset($queryvars[session_name()]);
    }

    reset($queryvars);
    $querystring = !GhCoreGlobals::$useModRewrite ? '?' : '';

    $separator = ini_get("arg_separator.output");

    for($i = 0; list($key, $value) = each($queryvars);)
    {
      if(GhCoreStrings::realTrim($value) != '')
      {
        if($i++ > 0)
          $querystring .= $separator;

        if($key == 'mod')
          $value = strtolower($value);

        $querystring .= (GhCoreGlobals::$useModRewrite && $key == 'mod')
                      ? GhCoreStrings::urlEncode($value) : $key . '=' . GhCoreStrings::urlEncode($value);
      }
    }

    return $querystring;
  }

  /**
   * Entfernt das Ampersand (&-Zeichen) aus einem String.
   *
   * Dies kann beispielsweise verwendet werden wenn ein Query-String in Text-E-Mails versendet werden soll.
   *
   * @param string $queryString Der zu filternde String.
   * @return string
   */
  public static function stripAmps($queryString)
  {
    return str_replace("&amp;", "&", $queryString);
  }

  /**
   * Liefert die URL der aktuellen Webseite (des aktuellen Scripts) zurück.
   *
   * @return string
   */
  public static function getScript()
  {
    if(isset($PHP_SELF))
      return $PHP_SELF;
    else if(getenv('PHP_SELF'))
      return getenv('PHP_SELF');
    else
      return getenv('SCRIPT_NAME');
  }

  /**
   * Liefert eine GET-Variable zurück.
   *
   * Es kann ein optionaler Vorgabewert angegeben werden, der zurückgegeben
   * wird falls die Variable nicht gefunden wird.
   * Zusätzlich kann festgelegt werden, ob der Vorgabewert in die Session
   * geschrieben werden soll (unabhängig davon, ob der Wert gefunden wurde).
   *
   * Der Name kann als Array-Schreibweise vorliegen und wird entsprechend
   * aufgelöst (z.B. 'mydata[section1][data2]').
   *
   * @param string  $name    Der Name der GET-Variable
   * @param string  $default Vorgabewert falls keine GET-Variable mit diesem Namen existiert
   * @param boolean $set     true: der Vorgabewert wird als GET-Variable gesetzt
   * @return string
   */
  public static function get($name, $default = null, $set = false)
  {
    return self::_getOrSetValue($_GET, $name, $default, $set);
  }

  /**
   * Liefert eine POST-Variable zurück.
   *
   * Es kann ein optionaler Vorgabewert angegeben werden, der zurückgegeben
   * wird falls die Variable nicht gefunden wird.
   * Zusätzlich kann festgelegt werden, ob der Vorgabewert in die Session
   * geschrieben werden soll (unabhängig davon, ob der Wert gefunden wurde).
   *
   * Der Name kann als Array-Schreibweise vorliegen und wird entsprechend
   * aufgelöst (z.B. 'mydata[section1][data2]').
   *
   * @param string  $name    Der Name der POST-Variable
   * @param string  $default Vorgabewert falls keine POST-Variable mit diesem Namen existiert
   * @param boolean $set     true: der Vorgabewert wird als POST-Variable gesetzt
   * @return string
   */
  public static function post($name, $default = null, $set = false)
  {
    return self::_getOrSetValue($_POST, $name, $default, $set);
  }

  /**
   * Liefert eine COOKIE-Variable zurück.
   *
   * Es kann ein optionaler Vorgabewert angegeben werden, der zurückgegeben
   * wird falls die Variable nicht gefunden wird.
   * Zusätzlich kann festgelegt werden, ob der Vorgabewert in die Session
   * geschrieben werden soll (unabhängig davon, ob der Wert gefunden wurde).
   *
   * @param string  $name    Der Name der COOKIE-Variable
   * @param string  $default Vorgabewert falls keine COOKIE-Variable mit diesem Namen existiert
   * @param boolean $set     true: der Vorgabewert wird als COOKIE-Variable gesetzt
   * @return string
   */
  public static function cookie($name, $default = null, $set = false)
  {
    if(isset($_COOKIE[$name]))
    {
      if(ini_get('magic_quotes_gpc') == '1')
        $value = stripslashes($_COOKIE[$name]);
      else
        $value = $_COOKIE[$name];
    }
    else
      $value = $default;

    if($set)
      $_COOKIE[$name] = $default;

    return $value;
  }

  /**
   * Liefert eine SESSION-Variable zurück.
   *
   * Es kann ein optionaler Vorgabewert angegeben werden, der zurückgegeben
   * wird falls die Variable nicht gefunden wird.
   * Zusätzlich kann festgelegt werden, ob der Vorgabewert in die Session
   * geschrieben werden soll (unabhängig davon, ob der Wert gefunden wurde).
   *
   * @param string  $name    Der Name der SESSION-Variable
   * @param string  $default Vorgabewert falls keine SESSION-Variable mit diesem Namen existiert
   * @param boolean $set     true: der Vorgabewert wird als SESSION-Variable gesetzt
   * @return string
   */
  public static function session($name, $default = null, $set = false)
  {
    $value = isset($_SESSION[$name])
           ? $_SESSION[$name]
           : $default;

    if($set)
      $_SESSION[$name] = $default;

    return $value;
  }

  /**
   * Erstellt eine neue Session.
   *
   * Diese Methode muss aufgerufen werden bevor Daten in die Session
   * geschrieben oder aus der Session gelesen werden. Alternativ kann mit der
   * Methode [functions]continueSession[/functions] eine bestehende Session
   * fortgeführt werden.
   *
   * @param string $name Session-Name (Voreinstellung: 'sid')
   * @param int    $cookieLifetime Gültigkeitsdauer des Session-Cookie in
   *   Sekunden (Voreinstellung: 0 = nur für die Dauer der Browser-Session
   *   gültig)
   * @param string $cookieDomain Domain und Pfad des Session-Cookie
   */
  public static function startNewSession($name = 'sid', $cookieLifetime = 0, $cookieDomain = null)
  {
    if(!$name)
      throw new GhCoreException('The session name must not be empty');

    if($cookieDomain !== null || $cookieLifetime !== null)
    {
      $cookieParams = null;

      if($cookieDomain === null)
        session_set_cookie_params($cookieLifetime);
      else
      {
        $pathIndex = strpos($cookieDomain, '/');

        if($pathIndex === false)
          $cookiePath = '/';
        else
        {
          $cookiePath = substr($cookieDomain, $pathIndex);
          $cookieDomain = substr($cookieDomain, 0, $pathIndex);
        }

        if(strlen($cookieDomain) > 0)
          session_set_cookie_params($cookieLifetime, $cookiePath, $cookieDomain);
        else
          session_set_cookie_params($cookieLifetime, $cookiePath);
      }
    }

    session_name($name);
    session_start();

    // Session ID explizit erzeugen lassen (und nicht die vom Client ggf. übermittelte verwenden):
    session_regenerate_id(true);

    // Die neue Session sicherheitshalber löschen und neu erstellen:
    session_destroy();
    $_SESSION = array();
    session_start();

    $cookieParams = session_get_cookie_params();
    setcookie($name, session_id(), $cookieParams['lifetime'], $cookieParams['path'], $cookieParams['domain'], $cookieParams['secure'], $cookieParams['httponly']);

    // Session-Daten aktualisieren um die Session-Lifetime zu erhalten:
    $_SESSION['ghSessionLastAccess'] = time();
  }

  /**
   * Führt eine bereits für diesen Client existierende Session fort.
   *
   * Optional kann eine neue Session erzeugt werden wenn für diesen Client
   * noch keine existiert. Dazu dienen die zusätzlichen Parameter nach dem
   * Session-Namen (siehe [functions]startNewSession[/functions]).
   *
   * Diese Methode muss aufgerufen werden bevor Daten in die Session
   * geschrieben oder aus der Session gelesen werden. Alternativ kann mit der
   * Methode [functions]startNewSession[/functions] eine neue Session erzeugt
   * werden.
   *
   * @param string  $name Session-Name (Voreinstellung: 'sid')
   * @param boolean $startSession true: neue Session starten falls noch keine
   *   existiert, false: keine neue Session starten
   * @param int     $cookieLifetime Gültigkeitsdauer des Session-Cookie in
   *   Sekunden (Voreinstellung: 0 = nur für die Dauer der Browser-Session
   *   gültig)
   * @param string  $cookieDomain Domain und Pfad des Session-Cookie
   * @return int
   */
  public static function continueSession($name = 'sid', $startSession = true, $cookieLifetime = 0, $cookieDomain = null)
  {
    if(!$name)
      throw new GhCoreException('The session name must not be empty');

    if(isset($_SESSION['ghSessionLastAccess']))
    {
      $lastAccess = $_SESSION['ghSessionLastAccess'];
      $_SESSION['ghSessionLastAccess'] = time();

      return $lastAccess;
    }

    $sessionId = self::getValue($name, null, 'CPG');
    $lastAccess = null;

    if($sessionId)
    {
      session_name($name);
      session_start();

      if(isset($_SESSION['ghSessionLastAccess']))
        $lastAccess = $_SESSION['ghSessionLastAccess'];
    }

    if($lastAccess)
    {
      $_SESSION['ghSessionLastAccess'] = time();

      return $lastAccess;
    }
    elseif($startSession)
    {
      if(isset($_SESSION))
        session_destroy();

      self::startNewSession($name, $cookieLifetime, $cookieDomain);

      return $_SESSION['ghSessionLastAccess'];
    }

    return 0;
  }

  /**
   * Beendet eine Session und entfernt die Session-Daten und den
   * Session-Cookie.
   */
  public static function destroySession()
  {
    if(!isset($_SESSION))
      session_start();

    $_SESSION = array();

    // Session-Cookie leeren und als abgelaufen markieren:
    $sessionParams = session_get_cookie_params();
    setcookie(session_name(), '', time() - 86400, $sessionParams['path'], $sessionParams['domain'], $sessionParams['secure'], $sessionParams['httponly']);

    session_destroy();
  }

  /**
   * Liefert Informationen über eine hochgeladene Datei zurück.
   *
   * Die Rückgabe ist ein Array mit Informationen zur hochgeladenen Datei:
   *
   * <dl>
   * <dt>name</dt>
   * <dd>Name der hochgeladenen Datei</dd>
   * <dt>type</dt>
   * <dd>MIME-Typ der hochgeladenen Datei</dd>
   * <dt>tmp_name</dt>
   * <dd>Dateipfad, unter dem der Inhalt der hochgeladenen Datei temporär
   * abgelegt wurde.</dd>
   * <dt>error</dt>
   * <dd>Fehlercode</dd>
   * <dt>size</dt>
   * <dd>Größe der hochgeladenen Datei (in Bytes)</dd>
   * </dl>
   *
   * Wurde keine Datei hochgeladen (Fehlercode 4), so liefert die Methode den
   * Wert false. Existiert kein Eintrag zu dem angegebenen Namen, so liefert
   * die Methode den Wert null.
   * Wurden mehrere Dateien unter dem angegebenen Namen hochgeladen, so liefert
   * die Methode einen Array mit je einem Array im oben angegebenen Format pro
   * hochgeladener Datei.
   *
   * @param string $name Der Name der FILES-Variable
   * @return array
   */
  public static function file($name)
  {
    // Sicherstellen, dass $name ein nicht-leerer String ist:
    if(!is_string($name) || empty($name))
      return null;

    // Array-Namen in Namen und Pfad auflösen:
    $name = str_replace(']', '', $name);
    $path = explode('[', $name);
    $name = array_shift($path);

    if(!isset($_FILES[$name]))
      return null;

    $fileInfo = $_FILES[$name];

    if(!is_array($fileInfo) || !isset($fileInfo['tmp_name']))
      return null;

    if(!empty($path))
    {
      $tmpInfo = $fileInfo;
      $fileInfo = array('tmp_name' => self::_getArrayPathValue($tmpInfo['tmp_name'], $path));

      if($fileInfo['tmp_name'] === null)
        return null;

      $fileInfo['name'] = self::_getArrayPathValue($tmpInfo['name'], $path);
      $fileInfo['type'] = self::_getArrayPathValue($tmpInfo['type'], $path);
      $fileInfo['error'] = self::_getArrayPathValue($tmpInfo['error'], $path);
      $fileInfo['size'] = self::_getArrayPathValue($tmpInfo['size'], $path);
    }

    if(isset($fileInfo['error']) && $fileInfo['error'] == 4)
      return false;

    if(isset($fileInfo['tmp_name']) && is_array($fileInfo['tmp_name']))
    {
      $files = array();

      for($i=0; $i<count($fileInfo['tmp_name']); $i++)
      {
        if(isset($fileInfo['error'][$i]) && $fileInfo['error'][$i] == 4)
          continue;

        $files[] = array(
          'tmp_name' => $fileInfo['tmp_name'][$i],
          'name' => isset($fileInfo['name'][$i]) ? $fileInfo['name'][$i] : null,
          'type' => isset($fileInfo['type'][$i]) ? $fileInfo['type'][$i] : null,
          'error' => isset($fileInfo['error'][$i]) ? $fileInfo['error'][$i] : null,
          'size' => isset($fileInfo['size'][$i]) ? $fileInfo['size'][$i] : null
        );
      }

      if(empty($files))
        return false;
      else
        $fileInfo = $files;
    }

    return $fileInfo;
  }

  /**
   * Liefert Daten aus einem mehrdimensionalen Array anhand eines Pfads.
   *
   * @param array $array Daten-Array
   * @param array $path  Pfad
   * @return mixed
   */
  static protected function _getArrayPathValue($array, $path)
  {
    if(!is_array($path) || !is_array($array))
      return null;

    $value = $array;
    foreach($path as $entry)
    {
      if(!isset($value[$entry]))
        return null;

      $value = $value[$entry];
    }

    return $value;
  }

  /**
   * Liefert einen Wert aus einem Array anhand eines Namens zurück.
   *
   * Der Name kann als Array-Schreibweise vorliegen und wird entsprechend
   * aufgelöst (z.B. 'mydata[section1][data2]').
   *
   * @param array   $array   Array mit Daten
   * @param string  $name    Name des gesuchten Wertes
   * @param mixed   $default Vorgabewert
   * @param boolean $set     true: Vorgabewert in dem Array setzen
   */
  static protected function _getOrSetValue(&$array, $name, $default, $set)
  {
    // Sicherstellen, dass $name ein nicht-leerer String ist:
    if(!is_string($name) || empty($name))
      return $default;

    // Array-Namen in Pfad und Namen auflösen:
    $name = str_replace(']', '', $name);
    $path = explode('[', $name);
    $name = array_pop($path);

    // Array anhand des Pfades durchlaufen:
    $tmpArray = &$array;
    foreach($path as $entry)
    {
      if(isset($tmpArray[$entry]) && is_array($tmpArray[$entry]))
        $tmpArray = &$tmpArray{$entry};
      elseif($set)
      {
        $tmpArray[$entry] = array();
        $tmpArray = &$tmpArray[$entry];
      }
      else
        return $default;
    }

    // Den gesuchten Wert zurückgeben und ggf. setzen:
    if(isset($tmpArray[$name]))
      $value = self::_removeMagicQuotes($tmpArray[$name]);

    if($set)
      $tmpArray[$name] = $default;

    if(!isset($value))
      $value = $default;

    return $value;
  }

  /**
   * Bereinigt Daten um Magic-Quotes.

   * Diese Methode arbeitet rekursiv und bereinigt so alle Elemente eines
   * Arrays.
   *
   * @param mixed $data Daten
   * @return mixed Um Magic-Quotes bereinigte Daten
   */
  static protected function _removeMagicQuotes($data)
  {
    if(ini_get('magic_quotes_gpc') == '1')
    {
      if(is_scalar($data))
        $data = stripslashes($data);
      elseif(is_array($data))
      {
        foreach($data as $key => $value)
          $data[$key] = self::_removeMagicQuotes($value);
      }
    }
    return $data;
  }
}