<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2018 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Hilfsklasse für Strings.
 *
 * @package GhCore
 * @subpackage strings
 * @category Hilfsklassen
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2018 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhCoreStrings
{
  /**
   * Entfernt alle nicht-druckbaren Zeichen am Anfang und Ende eines Strings.
   *
   * @param string $string String
   * @return string
   */
  static public function realTrim($string)
  {
    return trim($string, "\x00..\x20");
  }

  /**
   * Prüft, ob ein String mit einem bestimmten Teilstring beginnt.
   *
   * @param string  $text       String
   * @param string  $prefix     Teilstring
   * @param boolean $ignoreCase Groß-/Kleinschreibung ignorieren
   * @return boolean
   */
  static public function startsWith($text, $prefix, $ignoreCase = false)
  {
    $text = (string)$text;
    $prefix = (string)$prefix;
    $prefixLengh = strlen($prefix);

    if(strlen($text) < $prefixLengh)
      return false;

    $subText = substr($text, 0, $prefixLengh);

    if($ignoreCase)
    {
      $subText = strtolower($subText);
      $prefix = strtolower($prefix);
    }

    return $subText === $prefix;
  }

  /**
   * Prüft, ob ein String mit einem bestimmten Teilstring endet.
   *
   * @param string  $text       String
   * @param string  $suffix     Teilstring
   * @param boolean $ignoreCase Groß-/Kleinschreibung ignorieren
   * @return boolean
   */
  static public function endsWith($text, $suffix, $ignoreCase = false)
  {
    $text = (string)$text;
    $suffix = (string)$suffix;
    $textLength = strlen($text);
    $suffixLength = strlen($suffix);

    if($textLength < $suffixLength)
      return false;

    $subText = substr($text, $textLength - $suffixLength);

    if($ignoreCase)
    {
      $subText = strtolower($subText);
      $suffix = strtolower($suffix);
    }

    return $subText === $suffix;
  }

  /**
   * Entfernt einen Teilstring vom Anfang eines Strings, sofern dieser mit dem
   * Teilstring beginnt.
   *
   * @param string  $text       String
   * @param string  $prefix     Teilstring
   * @param boolean $ignoreCase Groß-/Kleinschreibung ignorieren
   * @return string
   */
  public static function removeFromStart($text, $prefix, $ignoreCase = false)
  {
    if(GhCoreStrings::startsWith($text, $prefix, $ignoreCase))
      $text = substr($text, strlen($prefix));

    return $text;
  }

  /**
   * Entfernt einen Teilstring vom Ende eines Strings, sofern dieser auf den
   * Teilstring endet.
   *
   * @param string  $text       String
   * @param string  $suffix     Teilstring
   * @param boolean $ignoreCase Groß-/Kleinschreibung ignorieren
   * @return string
   */
  public static function removeFromEnd($text, $suffix, $ignoreCase = false)
  {
    if(GhCoreStrings::endsWith($text, $suffix, $ignoreCase))
      $text = substr($text, 0, strlen($text) - strlen($suffix));

    return $text;
  }

  /**
   * Encodiert einen String als URL.
   *
   * @param string $url URL
   * @return string
   */
  static function urlEncode($url)
  {
    $url = urlencode($url);

    // Apache mod_rewrite decodes the url once, so it must be encoded twice:
    if(defined('USE_MOD_REWRITE') && USE_MOD_REWRITE)
      $url = urlencode($url);

    return $url;
  }

  /**
   * Maskiert Sonderzeichen in einem String, so dass das Ergebnis als
   * Attributswert verwendet werden kann.
   *
   * Optional kann ein Array mit Schlüssel-Wert-Paaren für Zeichenersetzungen
   * angegeben werden. Die Voreinstellung ist, dass Anführungszeichen,
   * Zeilenumbruch, Wagenrücklauf und Tabulator durch einen vorangestellten
   * Backslash maskiert werden, so dass das Ergebnis syntaktisch korrekt in
   * Anführungszeichen (ohne Zeilenumbrüche) eingeschlossen werden kann.
   *
   * @param string $value String
   * @param array  $replacements Textersetzungen
   * @return string
   */
  static public function attributeValue($value, $replacements = null)
  {
    if(!is_array($replacements))
      $replacements = array('"' => '&quot;', "\n" => '\\n', "\r" => '\\r', "\t" => '\\t' );

    return str_replace(array_keys($replacements), array_values($replacements), $value);
  }

  /**
   * Erzeugt einen String im JSON Format für ein Schlüssel-Wert-Paar.
   *
   * Wird kein Datentyp angegeben, so wird dieser aus dem Wert ermittelt. Kann
   * kein Typ (boolean, int, double, float, string, array, object) ermittelt
   * werden, so wird als Wert null verwendet.
   *
   * Beispiel: myProperty:123 oder myStringProperty:"abc".
   *
   * @param string $property Name der Eigenschaft (des Schlüssels)
   * @param mixed  $value    Wert
   * @param string $type     Datentyp
   * @return string
   */
  static public function jsonProperty($property, $value, $type = null)
  {
    return '\'' . $property . '\':' . self::jsonValue($value, $type);
  }

  /**
   * Erzeugt einen String im JSON Format für einen Wert.
   *
   * Wird kein Datentyp angegeben, so wird dieser aus dem Wert ermittelt. Kann
   * kein Typ (boolean, int, double, float, string, array, object) ermittelt
   * werden, so wird als Wert null verwendet.
   *
   * @param mixed  $value Wert
   * @param string $type  Datentyp
   * @return string
   */
  static public function jsonValue($value, $type = null)
  {
    if($type === null)
    {
      if($value === true || $value === false)
        $type = 'boolean';
      elseif(is_int($value))
        $type = 'int';
      elseif(is_double($value))
        $type = 'double';
      elseif(is_float($value))
        $type = 'float';
      elseif(is_string($value))
        $type = 'string';
      elseif(is_array($value))
        $type = 'array';
      elseif(is_object($value))
        $type = 'object';
    }

    $javascript = '';

    switch($type)
    {
      case 'boolean':
      case 'bool':
        $javascript .= $value ? 'true' : 'false';
        break;
      case 'int':
      case 'integer':
        $javascript .= (int)$value;
        break;
      case 'float':
        $javascript .= (float)$value;
        break;
      case 'double':
        $javascript .= (double)$value;
        break;
      case 'string':
        $javascript .= '"' . self::attributeValue($value, array('\\' => '\\\\', '"' => '\\"', "\n" => '\\n', "\r" => '\\r', "\t" => '\\t' )) . '"';
        break;
      case 'array':
        if(is_array($value))
        {
          $values = array();

          foreach($value as $arrayValue)
            $values[] = self::jsonValue($arrayValue);

          $javascript .= '[' . implode(',', $values) . ']';
        }
        elseif(is_object($value))
        {
          $values = array();

          foreach(get_object_vars($value) as $objectValue)
            $values[] = self::jsonValue($objectValue);

          $javascript .= '[' . implode(',', $values) . ']';
        }
        else
          $javascript .= (string)$value;
        break;
      case 'object':
        if(is_object($value))
        {
          $values = array();
          foreach(get_object_vars($value) as $objectProperty => $objectValue)
            $values[] = self::jsonProperty($objectProperty, $objectValue);
          $javascript .= '{' . implode(',', $values) . '}';
        }
        elseif(is_array($value))
        {
          $values = array();
          foreach($value as $arrayKey => $arrayValue)
            $values[] = self::jsonProperty($arrayKey, $arrayValue);
          $javascript .= '{' . implode(',', $values) . '}';
        }
        else
          $javascript .= (string)$value;
        break;
      case 'var':
      case 'variable':
      case 'function':
        $javascript .= (string)$value;
          break;
      default:
        $javascript .= 'null';
    }

    return $javascript;
  }

  /**
   * Liefert den Inhalt des 'body' Knotens eines HTML Texts.
   *
   * @param string $html HTML Text
   * @return string
   */
  static public function getHtmlBody($html)
  {
    $startIndex = stripos($html, '<body');

    if($startIndex !== false)
    {
      $startIndex = strpos($html, '>', $startIndex);

      if($startIndex !== false)
      {
        $endIndex = stripos($html, '</body', $startIndex);

        if($endIndex !== false)
          $html = substr($html, $startIndex + 1, $endIndex - $startIndex - 1);
      }
    }

    return $html;
  }

  /**
   * Ersetzt den Inhalt des 'body' Knotens eines HTML Texts.
   *
   * @param string $html HTML Text
   * @param string $body neuer Inhalt für den 'body' Knoten
   * @return string
   */
  static public function replaceHtmlBody($html, $body)
  {
    $startIndex = stripos($html, '<body');

    if($startIndex !== false)
    {
      $startIndex = strpos($html, '>', $startIndex);

      if($startIndex !== false)
      {
        $endIndex = stripos($html, '</body', $startIndex);

        if($endIndex !== false)
          $html = substr($html, 0, $startIndex + 1) . $body . substr($html, $endIndex);
      }
    }

    return $html;
  }
}