<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2018 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Hilfsfunktionen für die RPC API.
 *
 * @package GhRpc
 * @subpackage core
 * @category PHP-RPC-Kernklassen
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2018 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhRpcUtils
{
  /**
   * Hilfskonstante, die ein Set darstellt, in dem alle Enums gesetzt sind.
   */
  const SET_ALL = -1;

  /**
   * Erstellt ein Set aus einer oder mehrerer Enum-Konstanten.
   *
   * Sets werden als Integer-Werte repräsentiert, bei denen jeder Enum-Wert
   * einem Bit entspricht. Mehrere Konstanten können entweder als zusätzliche
   * Parameter oder als Array übergeben werden.
   *
   * Beispiel:
   * <br/>
   * Erzeugen eines Sets für die Objekt-Typen E-Mail, Notiz und Telefonnotiz:
   * <br/>
   * <code>
   * $kinds = GhRpcUtils::newSet( rpc_itemslib::ikEmail, rpc_itemslib::ikNote, rpc_itemslib::ikCall );
   * </code>
   *
   * @param int|array $enum     Enum-Konstante, oder ein Array mit Enum-Konstanten
   * @param int       $enum,... Weitere Enum-Konstanten
   * @return int Set
   */
  public static function newSet($enum)
  {
    if(!is_array($enum))
      $enum = array($enum);

    for($i = 1; $i < func_num_args(); $i++)
    {
      $param = func_get_arg($i);

      if(is_numeric($param))
        $enum[] = $param;
    }

    $set = 0;

    foreach($enum as $enum)
      $set |= (1 << $enum);

    return $set;
  }

  /**
   * Fügt eine oder mehrere Enum-Konstanten einem Set hinzu.
   *
   * Sets werden durch Integer-Werte repräsentiert, bei denen die
   * Enum-Konstanten jeweils ein bestimmtes Bit darstellen. Mehrere Konstanten
   * können entweder als zusätzliche Parameter oder als Array übergeben werden.
   *
   * @param int       $set       Set
   * @param array|int $enums     Enum-Konstante (oder ein Array mit Enum-Konstanten)
   * @param int       $enums,... Weitere Enum-Konstanten
   * @return int Set
   */
  public static function addToSet($set, $enums)
  {
    $enumsArray = func_get_args();
    array_shift($enumsArray);

    if(is_array($enums))
    {
      array_shift($enumsArray);
      $enumsArray = array_merge($enumsArray, $enums);
    }

    $enumSet = call_user_func_array(array('self', 'newSet'), $enumsArray);

    return $set | $enumSet;
  }

  /**
   * Entfernt eine oder mehrere Enum-Konstanten aus einem Set.
   *
   * Sets werden durch Integer-Werte repräsentiert, bei denen die
   * Enum-Konstanten jeweils ein bestimmtes Bit darstellen. Mehrere Konstanten
   * können entweder als zusätzliche Parameter oder als Array übergeben werden.
   *
   * @param int       $set       Set
   * @param array|int $enums     Enum-Konstante (oder ein Array mit Enum-Konstanten)
   * @param int       $enums,... Weitere Enum-Konstanten
   * @return int Set
   */
  public static function removeFromSet($set, $enums)
  {
    $enumsArray = func_get_args();
    array_shift($enumsArray);

    if(is_array($enums))
    {
      array_shift($enumsArray);
      $enumsArray = array_merge($enumsArray, $enums);
    }

    $enumSet = call_user_func_array(array('self', 'newSet'), $enumsArray);

    return $set & ~$enumSet;
  }

  /**
   * Prüft, ob bestimmte Enum-Konstanten in einem Set enthalten sind.
   *
   * Sets werden durch Integer-Werte repräsentiert, bei denen die
   * Enum-Konstanten jeweils ein bestimmtes Bit darstellen. Mehrere Konstanten
   * können entweder als zusätzliche Parameter oder als Array übergeben werden.
   *
   * @param int       $set       Set
   * @param array|int $enums     Enum-Konstante (oder ein Array mit Enum-Konstanten)
   * @param int       $enums,... Weitere Enum-Konstanten
   * @return int
   */
  public static function inSet($set, $enums)
  {
    $enumsArray = func_get_args();
    array_shift($enumsArray);

    if(is_array($enums))
    {
      array_shift($enumsArray);
      $enumsArray = array_merge($enumsArray, $enums);
    }

    $enumSet = call_user_func_array(array('self', 'newSet'), $enumsArray);

    return ($set & $enumSet) == $enumSet;
  }
}